# -*- coding: utf-8 -*-
"""This module contains the controller classes of the application."""

# symbols which are imported by "from wiki20.controllers import *"
__all__ = ['Root']

# standard library imports
import logging
import re

# third-party imports
from docutils.core import publish_parts

# TurboGears imports
from turbogears import controllers, expose, flash, redirect, url

# project specific imports
from wiki20.model import Page

log = logging.getLogger("wiki20.controllers")

wikiwords = re.compile(r"\b([A-Z]\w+[A-Z]+\w+)")


class Root(controllers.RootController):
    """The root controller of the application."""

    @expose("wiki20.templates.page")
    def index(self , pagename="FrontPage"):
        page = Page.get_by(pagename=pagename)
        if not page:
            raise redirect("notfound", pagename=pagename)
        content = publish_parts(page.data,
            writer_name="html")['html_body']
        content = wikiwords.sub(r'<a href="%s\1">\1</a>' % url('/'), content)
        return dict(data=content, page=page)

    @expose("wiki20.templates.edit")
    def notfound(self, pagename):
        page = Page(pagename=pagename, data="")
        return dict(page=page)

    @expose("wiki20.templates.edit")
    def edit(self, pagename):
        page = Page.get_by(pagename=pagename)
        return dict(page=page)

    @expose("wiki20.templates.pagelist")
    @expose("json")
    def pagelist(self):
        print repr(Page.query)
        pages = [page.pagename for page in Page.query.order_by(Page.pagename)]
        return dict(pages=pages)

    @expose()
    def save(self, pagename, data, submit):
        page = Page.get_by(pagename=pagename)
        page.data = data
        flash("Changes saved!")
        raise redirect("/%s" % pagename)

    @expose()
    def default(self, pagename):
        return self.index(pagename)